-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftCastBarText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Cast Bar Text"] = true,
	["Cast bar text options"] = true,
	["Enable"] = true,
	["Show cast bar text"] = true,
	["Font"] = true, 
	["Sets the font for cast bar text"] = true,
	["Font Size"] = true,
	["Sets the font height of the cast bar text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on cast bar text"] = true,
	["Alignment"] = true,
	["Sets the alignment of the cast bar text"] = true,
	["Outline"] = true,
	["Sets the font outline for cast bar text"] = true,
	["Position"] = true,
	["Adjust cast bar text position"] = true,
	["Left Offset"] = true,
	["Sets the offset of the left of the text"] = true,
	["Right Offset"] = true,
	["Sets the offset of the right of the text"] = true,
	["Vertical Offset"] = true,
	["Sets the vertical offset of the text"] = true,
	["Color"] = true,
	["Sets the cast bar text color"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
} end)

-----------------------------------------------------------------------------

local castBarTextRegion = nil

-----------------------------------------------------------------------------

local AloftCastBarText = Aloft:NewModule("CastBarText", "AceEvent-2.0")
AloftCastBarText.db = Aloft:AcquireDBNamespace("castBarText")
Aloft:RegisterDefaults("castBarText", "profile", {
	enable			= true,
	font			= "Arial Narrow",
	fontSize		= 9,
	shadow			= true,
	alignment		= "LEFT",
	outline			= "",
	offsets =
	{
		left		= 0,
		right		= 0,
		vertical	= 0,
	},
	color			= { 1, 1, 1, 1 }
})

Aloft.Options.args.castBarText =
{
	type = 'group',
	name = L["Cast Bar Text"],
	desc = L["Cast bar text options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show cast bar text"],
			get = function() return AloftCastBarText.db.profile.enable end,
			set = function(v) AloftCastBarText.db.profile.enable = v AloftCastBarText:Update() end,
			order = 1
		},
		font =
		{
			type = "text", 
			name = L["Font"], 
			desc = L["Sets the font for cast bar text"],
			disabled = function() return not AloftCastBarText.db.profile.enable end,
			get = function() return AloftCastBarText.db.profile.font end,
			set = function(v) AloftCastBarText.db.profile.font = v AloftCastBarText:Update() end,
			validate = SML_font,
		},
		fontSize =
		{
			type = 'range',
			name = L["Font Size"],
			desc = L["Sets the font height of the cast bar text"],
			max = 16,
			min = 5,
			step = 1,
			disabled = function() return not AloftCastBarText.db.profile.enable end,
			get = function() return AloftCastBarText.db.profile.fontSize end,
			set = function(value) AloftCastBarText.db.profile.fontSize = value AloftCastBarText:Update() end
		},
		shadow =
		{
			type = 'toggle',
			name = L["Font Shadow"],
			desc = L["Show font shadow on cast bar text"],
			disabled = function() return not AloftCastBarText.db.profile.enable end,
			get = function() return AloftCastBarText.db.profile.shadow end,
			set = function(v) AloftCastBarText.db.profile.shadow = v AloftCastBarText:Update() end
		},
		alignment =
		{
			type = 'text',
			name = L["Alignment"],
			desc = L["Sets the alignment of the cast bar text"],
			disabled = function() return not AloftCastBarText.db.profile.enable end,
			get = function() return AloftCastBarText.db.profile.alignment end,
			set = function(value) AloftCastBarText.db.profile.alignment = value AloftCastBarText:Update() end,
			validate = {"LEFT", "CENTER", "RIGHT" },
		},
		outline =
		{
			type = 'text',
			name = L["Outline"],
			desc = L["Sets the font outline for cast bar text"],
			disabled = function() return not AloftCastBarText.db.profile.enable end,
			get = function() return AloftCastBarText.db.profile.outline end,
			set = function(value) AloftCastBarText.db.profile.outline = value AloftCastBarText:Update() end,
			validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust cast bar text position"],
			disabled = function() return not AloftCastBarText.db.profile.enable end,
			args =
			{
				left =
				{
					type = 'range',
					name = L["Left Offset"],
					desc = L["Sets the offset of the left of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarText.db.profile.offsets.left end,
					set = function(v) AloftCastBarText.db.profile.offsets.left = v AloftCastBarText:Update() end
				},
				right =
				{
					type = 'range',
					name = L["Right Offset"],
					desc = L["Sets the offset of the right of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarText.db.profile.offsets.right end,
					set = function(v) AloftCastBarText.db.profile.offsets.right = v AloftCastBarText:Update() end
				},
				vertical =
				{
					type = 'range',
					name = L["Vertical Offset"],
					desc = L["Sets the vertical offset of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarText.db.profile.offsets.vertical end,
					set = function(v) AloftCastBarText.db.profile.offsets.vertical = v AloftCastBarText:Update() end
				},
			},
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the cast bar text color"],
			disabled = function() return not AloftCastBarText.db.profile.enable end,
			get = function() return unpack(AloftCastBarText.db.profile.color) end,
			set = function(r, g, b, a) AloftCastBarText.db.profile.color = { r, g, b, a } AloftCastBarText:Update() end,
			hasAlpha = true
		},
	},
}

-----------------------------------------------------------------------------

function AloftCastBarText:Update()
	self:CreateFrame()
	self:RegisterEvents()
	
	if self.db.profile.enable then
		for aloftData in Aloft:IterateVisibleNameplates() do
			self:SetupFrame(aloftData)
		end
	end
end

-----------------------------------------------------------------------------

function AloftCastBarText:CreateFrame()
	if not self.db.profile.enable then
		if castBarTextRegion then
			castBarTextRegion:Hide()
		end
		return
	end

	if not castBarTextRegion then
		castBarTextRegion = UIParent:CreateFontString()
	end
	
	self:PrepareText(castBarTextRegion, self.db.profile)

end

function AloftCastBarText:RegisterEvents()
	if self.db.profile.enable then	
		self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
		self:RegisterEvent("Aloft:OnCastBarValueChanged", "UpdateText")
		self:RegisterEvent("Aloft:OnCastBarShow", "OnCastBarShow")
		self:RegisterEvent("Aloft:OnCastBarHide", "OnCastBarHide")
	else
		self:CancelEvent("Aloft:SetupFrame")
		self:CancelEvent("Aloft:OnCastBarValueChanged")
		self:CancelEvent("Aloft:OnCastBarShow")
		self:CancelEvent("Aloft:OnCastBarHide")
	end
end

-----------------------------------------------------------------------------

function AloftCastBarText:GetTargetSpellName()
	local _, _, displayName = UnitCastingInfo("target")
	if displayName then
		return displayName
	end
	_, _, displayName = UnitChannelInfo("target")
	if displayName then
		return displayName
	end
	return ""
end

-----------------------------------------------------------------------------

function AloftCastBarText:OnEnable()
	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:Update()
		end
	end)
	
	self:Update()
end

function AloftCastBarText:OnDisable()
	self:UnregisterAllEvents()
	
	if castBarTextRegion then
		castBarTextRegion:Hide()
	end
end

function AloftCastBarText:SetupFrame(aloftData)
	if aloftData:IsCurrentTarget() then
		self:OnCastBarShow(aloftData)
	end	
end

function AloftCastBarText:UpdateText(aloftData)
	local castBar = aloftData.castBar
	
	if aloftData.castBarStartTime ~= castBar:GetMinMaxValues() then
		castBarTextRegion:SetText(self:GetTargetSpellName())
	end
end

function AloftCastBarText:OnCastBarShow(aloftData)
	local castBar = aloftData.castBar
	local profile = self.db.profile

	castBarTextRegion:SetText(self:GetTargetSpellName())
	castBarTextRegion:ClearAllPoints()
	castBarTextRegion:SetPoint("TOPLEFT", castBar, "TOPLEFT", profile.offsets.left, profile.offsets.vertical)
	castBarTextRegion:SetPoint("BOTTOMRIGHT", castBar, "BOTTOMRIGHT", profile.offsets.right, profile.offsets.vertical)
	castBarTextRegion:SetAlpha(1.0)
	castBarTextRegion:SetParent(aloftData.UIFrame)
	castBarTextRegion:Show()

	aloftData.castBarStartTime = castBar:GetMinMaxValues()

	self:ScheduleRepeatingEvent("AloftCastBarText:UpdateAlpha", self.UpdateAlpha, 0, self, aloftData.spellIconRegion)
end

function AloftCastBarText:OnCastBarHide(aloftData)
	self:CancelScheduledEvent("AloftCastBarText:UpdateAlpha")
	castBarTextRegion:Hide()
end

function AloftCastBarText:UpdateAlpha(spellIconRegion)
	castBarTextRegion:SetAlpha(spellIconRegion:GetAlpha())
end

-----------------------------------------------------------------------------
