-----------------------------------------------------------------------------

if select(2, UnitClass("player")) ~= "ROGUE" and select(2, UnitClass("player")) ~= "DRUID" then
	return
end

-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftComboPointsText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Combo Points Text"] = true,
	["Combo points text options"] = true,
	["Enable"] = true,
	["Show combo points text"] = true,
	["Font"] = true, 
	["Sets the font for combo points text"] = true,
	["Font Size"] = true,
	["Sets the font height of the combo points text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on combo points text"] = true,
	["Outline"] = true,
	["Sets the font outline for combo points text"] = true,
	["Anchor"] = true, 
	["Sets the anchor for the combo points text"] = true,
	["Anchor To"] = true, 
	["Sets the relative point on the health bar to anchor the combo points text"] = true,
	["X Offset"] = true,
	["X offset of the combo points text"] = true,
	["Y Offset"] = true,
	["Y offset of the combo points text"] = true,
	["Color"] = true,
	["Sets the combo points text color"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
} end)

-----------------------------------------------------------------------------

local comboPointsTextRegion = nil
local targetNameplate = nil

-----------------------------------------------------------------------------

local AloftComboPointsText = Aloft:NewModule("ComboPointsText", "AceEvent-2.0")
AloftComboPointsText.db = Aloft:AcquireDBNamespace("comboPointsText")
Aloft:RegisterDefaults("comboPointsText", "profile", {
	enable			= true,
	font			= "Arial Narrow",
	fontSize		= 16,
	shadow			= true,
	outline			= "THICKOUTLINE",
	point			= "RIGHT",
	relativeToPoint	= "LEFT",
	offsetX			= 0,
	offsetY			= 0,
	color			= { 1, 1, 1, 1 }
})

Aloft.Options.args.comboPointsText =
{
	type = 'group',
	name = L["Combo Points Text"],
	desc = L["Combo points text options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show combo points text"],
			get = function() return AloftComboPointsText.db.profile.enable end,
			set = function(v) AloftComboPointsText.db.profile.enable = v AloftComboPointsText:Update() end,
			order = 1
		},
		font =
		{
			type = "text", 
			name = L["Font"], 
			desc = L["Sets the font for combo points text"],
			disabled = function() return not AloftComboPointsText.db.profile.enable end,
			get = function() return AloftComboPointsText.db.profile.font end,
			set = function(v) AloftComboPointsText.db.profile.font = v AloftComboPointsText:Update() end,
			validate = SML_font,
		},
		fontSize =
		{
			type = 'range',
			name = L["Font Size"],
			desc = L["Sets the font height of the combo points text"],
			max = 16,
			min = 5,
			step = 1,
			disabled = function() return not AloftComboPointsText.db.profile.enable end,
			get = function() return AloftComboPointsText.db.profile.fontSize end,
			set = function(value) AloftComboPointsText.db.profile.fontSize = value AloftComboPointsText:Update() end
		},
		shadow =
		{
			type = 'toggle',
			name = L["Font Shadow"],
			desc = L["Show font shadow on combo points text"],
			disabled = function() return not AloftComboPointsText.db.profile.enable end,
			get = function() return AloftComboPointsText.db.profile.shadow end,
			set = function(v) AloftComboPointsText.db.profile.shadow = v AloftComboPointsText:Update() end
		},
		outline =
		{
			type = 'text',
			name = L["Outline"],
			desc = L["Sets the font outline for combo points text"],
			disabled = function() return not AloftComboPointsText.db.profile.enable end,
			get = function() return AloftComboPointsText.db.profile.outline end,
			set = function(value) AloftComboPointsText.db.profile.outline = value AloftComboPointsText:Update() end,
			validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
		},
		anchor = {
			type = "text", 
			name = L["Anchor"], 
			desc = L["Sets the anchor for the combo points text"],
			disabled = function() return not AloftComboPointsText.db.profile.enable end,
			get = function() return AloftComboPointsText.db.profile.point end,
			set = function(v) AloftComboPointsText.db.profile.point = v AloftComboPointsText:Update() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		anchorto = {
			type = "text", 
			name = L["Anchor To"], 
			desc = L["Sets the relative point on the health bar to anchor the combo points text"],
			disabled = function() return not AloftComboPointsText.db.profile.enable end,
			get = function() return AloftComboPointsText.db.profile.relativeToPoint end,
			set = function(v) AloftComboPointsText.db.profile.relativeToPoint = v AloftComboPointsText:Update() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		offsetX =
		{
			type = 'range',
			name = L["X Offset"],
			desc = L["X offset of the combo points text"],
			min = -32,
			max = 32,
			step = 1,
			disabled = function() return not AloftComboPointsText.db.profile.enable end,
			get = function() return AloftComboPointsText.db.profile.offsetX end,
			set = function(v) AloftComboPointsText.db.profile.offsetX = v AloftComboPointsText:Update() end
		},
		offsetY =
		{
			type = 'range',
			name = L["Y Offset"],
			desc = L["Y offset of the combo points text"],
			min = -32,
			max = 32,
			step = 1,
			disabled = function() return not AloftComboPointsText.db.profile.enable end,
			get = function() return AloftComboPointsText.db.profile.offsetY end,
			set = function(v) AloftComboPointsText.db.profile.offsetY = v AloftComboPointsText:Update() end
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the combo points text color"],
			disabled = function() return not AloftComboPointsText.db.profile.enable end,
			get = function() return unpack(AloftComboPointsText.db.profile.color) end,
			set = function(r, g, b, a) AloftComboPointsText.db.profile.color = { r, g, b, a } AloftComboPointsText:Update() end,
			hasAlpha = true
		},
	},
}

-----------------------------------------------------------------------------

function AloftComboPointsText:Update()
	self:CreateFrame()
	self:RegisterEvents()
	
	if self.db.profile.enable then
		if targetNameplate then
			local healthBar = targetNameplate.healthBar
			local profile = self.db.profile
			self:PlaceFrame(comboPointsTextRegion, healthBar, self.db.profile)
		end
	end
end

-----------------------------------------------------------------------------

function AloftComboPointsText:CreateFrame()
	if not self.db.profile.enable then
		if comboPointsTextRegion then
			comboPointsTextRegion:Hide()
		end
		return
	end

	if not comboPointsTextRegion then
		comboPointsTextRegion = UIParent:CreateFontString()
	end
	
	self:PrepareText(comboPointsTextRegion, self.db.profile)
end

function AloftComboPointsText:RegisterEvents()
	if self.db.profile.enable then
		self:RegisterEvent("PLAYER_TARGET_CHANGED", "UpdateTarget")
		self:RegisterEvent("PLAYER_COMBO_POINTS", "UpdateText")
		self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
		self:RegisterEvent("Aloft:OnNameplateHide", "OnNameplateHide")
	else
		self:CancelEvent("PLAYER_TARGET_CHANGED")
		self:CancelEvent("PLAYER_COMBO_POINTS")
		self:CancelEvent("Aloft:OnNameplateShow")
		self:CancelEvent("Aloft:OnNameplateHide")
	end
end

-----------------------------------------------------------------------------

function AloftComboPointsText:OnEnable()	
	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:Update()
		end
	end)
	
	self:Update()
end

function AloftComboPointsText:OnDisable()
	self:UnregisterAllEvents()
	
	if comboPointsTextRegion then
		comboPointsTextRegion:Hide()
	end
end

function AloftComboPointsText:OnNameplateShow(aloftData)
	if not targetNameplate and aloftData:IsCurrentTarget() then
		targetNameplate = aloftData
	
		local healthBar = aloftData.healthBar
		local profile = self.db.profile
	
		self:PlaceFrame(comboPointsTextRegion, healthBar, self.db.profile)
		comboPointsTextRegion:SetParent(aloftData.UIFrame)
		comboPointsTextRegion:Show()
	end
end

function AloftComboPointsText:OnNameplateHide(aloftData)
	if targetNameplate == aloftData then
		targetNameplate = nil
		comboPointsTextRegion:Hide()
	end
end

function AloftComboPointsText:UpdateText()
	local comboPoints = GetComboPoints()

	local newTargetNameplate = Aloft:GetTargetNameplate()
	
	if newTargetNameplate and newTargetNameplate ~= targetNameplate then
		targetNameplate = newTargetNameplate
		local healthBar = targetNameplate.healthBar
		local profile = self.db.profile
	
		self:PlaceFrame(comboPointsTextRegion, healthBar, self.db.profile)
		comboPointsTextRegion:SetParent(targetNameplate.UIFrame)
	end

	if comboPoints > 0 then
		comboPointsTextRegion:SetText(comboPoints)
		comboPointsTextRegion:Show()
	else
		comboPointsTextRegion:SetText("")
		comboPointsTextRegion:Hide()
	end
end

function AloftComboPointsText:UpdateTarget()
	if GetComboPoints() > 0 then 
		self:UpdateText()
	end
end

-----------------------------------------------------------------------------
